/*
 * Copyright (c) 2016 Chris Johns <chrisj@rtems.org>.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Start the BSD stack and run rc.conf the wait in a shell.
 */

#include <errno.h>
#include <stddef.h>
#include <stdio.h>

#include <rtems.h>
#include <rtems/bsd/bsd.h>
#include <rtems/console.h>
#include <rtems/printer.h>
#include <rtems/shell.h>
#include <rtems/untar.h>

#include <machine/rtems-bsd-rc-conf.h>

extern rtems_shell_cmd_t rtems_shell_DBSERVER_Command;

extern int _binary_rootfs_tar_start;
extern int _binary_rootfs_tar_size;

extern void bbapi_init(void);

#if __i386__
#include <bsp.h>

static int
rtems_shell_main_bsp_irq(int argc, char *argv[])
{
  uint32_t total = BSP_irq_count_dump(stdout);
  printf("Total: %lu\n", total);
  return 0;
}

rtems_shell_cmd_t rtems_shell_BSP_IRQ_Command = {
  "bsp-irq",                                /* name */
  "bsp-irq",                                /* usage */
  "misc",                                   /* topic */
  rtems_shell_main_bsp_irq,                 /* command */
  NULL,                                     /* alias */
  NULL,                                     /* next */
  0755,
  0,
  0
};
#endif

static void
expand_rootfs_tarfile(void)
{
  rtems_status_code sc;
  rtems_printer     printer;

  rtems_print_printer_printf(&printer);

  sc = Untar_FromMemory_Print((void *)(&_binary_rootfs_tar_start),
                              (size_t)&_binary_rootfs_tar_size,
                              &printer);
  if (sc != RTEMS_SUCCESSFUL)
    fprintf(stderr, "error: untar failed: %s\n", rtems_status_text(sc));
}

int
main(int argc, char** argv)
{
  rtems_status_code   sc;
  rtems_task_priority prioirity = 0;
  int                 r;

#if i386
  bbapi_init();
#endif

  expand_rootfs_tarfile();

  sc = rtems_task_set_priority(RTEMS_SELF, RTEMS_MAXIMUM_PRIORITY - 1U, &prioirity);
  if (sc != RTEMS_SUCCESSFUL)
    fprintf(stderr, "error: priority change failed: %s\n", rtems_status_text(sc));

  printf("BSD Initialisation ...\n");

  rtems_bsd_initialize();

  r = rtems_bsd_run_etc_rc_conf(15, false);
  if (r < 0)
    fprintf(stderr, "error: loading BSD /etc/rc.conf failed: %s\n", strerror(errno));

  rtems_shell_add_cmd_struct(&rtems_shell_DBSERVER_Command);
#if __i386__
  rtems_shell_add_cmd_struct(&rtems_shell_BSP_IRQ_Command);
#endif

  sc = rtems_shell_init("SHLL",
                        32 * 1024,
                        1,
                        CONSOLE_DEVICE_NAME,
                        false,
                        true,
                        NULL);
  if (sc != RTEMS_SUCCESSFUL)
    fprintf(stderr, "error: shell init: %s\n", rtems_status_text(sc));

  return 0;
}
